@@ -31,7 +31,8 @@ MultiBrain2 <- R6::R6Class(
     initialize = function(..., .list = NULL,
                           template_surface_types = NULL,
                           template_subject = getOption('threeBrain.template_subject', 'N27'),
-                          template_dir = getOption('threeBrain.template_dir', '~/rave_data/others/three_brain')){
+                          template_dir = getOption('threeBrain.template_dir', '~/rave_data/others/three_brain'),
+                          use_cache = TRUE, use_141 = getOption('threeBrain.use141', TRUE) ){
 
 
       l = unlist( c(list(...), .list) )
@@ -49,15 +50,23 @@ MultiBrain2 <- R6::R6Class(
       if( is.null(self$template_object) ){
         self$alter_template( template_subject = template_subject,
                              surface_types = template_surface_types,
-                             template_dir = template_dir )
+                             template_dir = template_dir,
+                             use_cache = use_cache, use_141 = use_141 )
       }
     },
 
     alter_template = function(surface_types = NULL,
                               template_subject = getOption('threeBrain.template_subject', 'N27'),
-                              template_dir = getOption('threeBrain.template_dir', '~/rave_data/others/three_brain')){
+                              template_dir = getOption('threeBrain.template_dir', '~/rave_data/others/three_brain'),
+                              use_cache = TRUE, use_141 = getOption('threeBrain.use141', TRUE)){
       # test
       template_path = file.path(template_dir, template_subject)
+
+      if( template_subject == 'N27' ){
+        check_freesurfer_path(template_path, autoinstall_template = TRUE)
+      }
+
+      # If N27, makesure it's installed
       stopifnot2(check_freesurfer_path(template_path),
                  msg = paste0('Cannot find template subject - ', template_subject,
                               '\nTo install N27 template subject, you can use:\n\n\t',
@@ -70,9 +79,9 @@ MultiBrain2 <- R6::R6Class(
         surface_types = unique(c('pial', unlist( surface_types )))
       }
 
-      self$template_object = freesurfer_brain(
+      self$template_object = freesurfer_brain2(
         fs_subject_folder = template_path, subject_name = template_subject,
-        additional_surfaces = surface_types)
+        surface_types = surface_types, use_cache = use_cache, use_141 = use_141)
     },
 
     add_subject = function(x){
@@ -130,6 +139,10 @@ MultiBrain2 <- R6::R6Class(
 
     },
 
+    set_electrodes = function( ... ){
+      self$template_object$set_electrodes( ... )
+    },
+
     set_electrode_values = function(table_or_path){
       stopifnot2(is.data.frame(table_or_path) || (length(table_or_path) == 1) && is.character(table_or_path),
                  msg = 'table_or_path must be either data.frame or path to a csv file')
